#!/usr/bin/env bash

# Echo commands and exit on first error
set -xe

# Everything in braces to synchronize stderr and stdout.
{

# https://stackoverflow.com/questions/5014823/how-to-profile-a-bash-shell-script-slow-startup
PS4='+ $EPOCHREALTIME\011 '

# Set SRC_DIR if necessary, e.g. when running outside CI.
SRC_DIR="${SRC_DIR-$(pwd)}"

# Enter ramdrive to prevent errors.
#cd /dev/shm

# Install ScopeSIM and Python-CPL, not yet a conda package.
# This needs to be here for the test to run in conda build.
# TODO: Ensure MicadoWISE works without these as well.
pip install skycalc_ipy
pip install "ScopeSIM==0.1.4"  # TODO: Replace with later ScopeSIM when possible.
pip install git+https://github.com/astronomyk/ScopeSim_Templates.git@hb/micadowise
# TODO: Fetch ScopeSim data package.

# Set CONDA_SP_DIR, should be set automatically if sitepackages-dir is available.
# TODO: Revert to using sitepackages-dir if that is public.
CONDA_SP_DIR="$(python -c 'from distutils.sysconfig import get_python_lib; print(get_python_lib())')"

coverage erase

# Do some non database tests without relying on common.
env \
    database_engine=filebased \
    database_user=AWANONYMOUS \
    mockcommon=True \
  coverage run --branch --append --source="${CONDA_SP_DIR}/micado" -m pytest \
    "${SRC_DIR}/src/micado/test/functional"

# TODO: Figure out a way to run all notebooks automatically.
#       Doing *.py will not always result in a positive exit code.
coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/DigitalDesignGuide.py
coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/SimulateRawScience.py
coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/SimulateFlat.py
coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/FullStandardImaging.py
coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/RecipeValidation.py

# Do integration tests, in particular CPL tests.
# These tests are not required to build the package, only in CI.
# These tests do not work with the database.
if [[ -n ${MICADO_INTEGRATION_TESTS:+x} ]]; then
    echo "MICADO_INTEGRATION_TESTS set, performing integration tests."
    # python-cpl is installed only when necessary
    pip install python-cpl
    env \
        database_engine=filebased \
        database_user=AWANONYMOUS \
    coverage run --branch --append --source="${CONDA_SP_DIR}/micado" -m pytest \
        "${SRC_DIR}/src/micado/test/integration"
else
    echo "MICADO_INTEGRATION_TESTS unset, skipping integration tests."
fi


if [[ -n "${CONDA_DOWNLOAD_KEY+x}" ]]; then
    conda config --prepend channels https://${CONDA_DOWNLOAD_KEY}@conda.astro-wise.org
    conda install -y common

    # Set database credentials.
    if [[ -n ${GITLAB_CI+x} ]]; then
        export \
            database_tablespacename="pg_default" \
            database_engine="postgresql" \
            database_port="5432" \
            database_name="postgres/${POSTGRES_DB}" \
            database_user="${POSTGRES_USER}" \
            database_password="${POSTGRES_PASSWORD}"
    fi

    if [[ ${database_engine:-filebased} != filebased ]]; then
        # Create database structures for all persistent classes.
        env \
            ask_administrator_password='' \
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" -m \
            micado.toolbox.backends.postgresql.dbtestsetup

        # First run some non database tests with the database engine loaded.
        # These can still detect problems with the database code.
        env \
            database_user="AWTEST" \
            database_password="lmno" \
        coverage run --branch --source="${CONDA_SP_DIR}/micado" -m pytest \
            "${SRC_DIR}/src/micado/test/functional"

        # Run tests that actually test the database.
        env \
            database_user="AWTEST" \
            database_password="lmno" \
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" -m pytest \
            "${SRC_DIR}/src/micado/test/database"

        # Also test the notebooks with database
        export database_user="AWTEST" database_password="lmno"
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/DigitalDesignGuide.py
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/SimulateRawScience.py
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/SimulateFlat.py
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/FullStandardImaging.py
        coverage run --branch --append --source="${CONDA_SP_DIR}/micado" "${SRC_DIR}"/notebooks/RecipeValidation.py

    fi
fi

# Generate coverage report.
coverage report -m

# Ignore E203 because black disagrees.
# https://black.readthedocs.io/en/stable/the_black_code_style.html#slices
# TODO: Reenable flake8
#flake8 --docstring-convention google --max-line-length 120 --max-doc-length 80 --extend-ignore=E203 "${CONDA_SP_DIR}/micado"

black --check "${CONDA_SP_DIR}/micado"

# Install sourcery. This might break things and is thus disabled for now.
#pip install sourcery-cli
# Login if SOURCERY_TOKEN (a masked CI variable) is set.
#if [[ -n ${SOURCERY_TOKEN+x} ]]; then
#    sourcery login --token "${SOURCERY_TOKEN}"
#fi
# Find changed files and pass them to Sourcery
# git diff master --name-only | xargs sourcery refactor --check
# For now, just run on the whole code base.
# Sourcery is disabled for now, because it makes it impossible for
# others to run conda-build.
# TODO: Re-enable sourcery for CI only.
# sourcery refactor --check "${CONDA_SP_DIR}/micado"

} 2>&1
