"""Create minipipeline.

This script creates darks and flats to test the first two recipes and their
integration into MicadoWISE.
"""

# Import all relevant classes. First a model of the relevant hardware.
from pathlib import Path

from micado.dataitems.Persistence import Persistence
from micado.hardware.ShotNoise import ShotNoise
from micado.main.ObservingBlock import ObservingBlock
from micado.hardware.Detector import Detector
from micado.hardware.DetectorArray import DetectorArray
from micado.hardware.DarkCurrent import DarkCurrent
from micado.hardware.SummedExposure import SummedExposure

# The relevant template.
from micado.templates.MicadoImgCalDark import MicadoImgCalDark

# The raw data.
from micado.dataitems.Dark import Dark
from micado.dataitems.Dark import DarkExtension

# The recipes.
from micado.recipes.McdDetDepersist import McdDetDepersist
from micado.recipes.McdDetDark import McdDetDark

# Processed data.
from micado.dataitems.DepersistedDark import DepersistedDark
from micado.dataitems.MasterDark import MasterDark
from micado.toolbox.oca_from_python import OCARules


def simulate_minipipeline():
    """Create Darks."""

    # First simulate the hardware.

    # The size of the detector in mm.
    # A small detector is used for this demonstration.
    # The full detector is 30.72 mm.
    height_width = 0.24

    # The (extraordinary high) value of the Dark current in e/s.
    dark_real = 100.0

    # Model a single Detector.
    mydetector = Detector(
        xhw=height_width,
        yhw=height_width,
    )
    mydetectorlist = [mydetector] * 9

    # Model the dark current.
    mydarkcurrent = DarkCurrent(value=dark_real)

    # Model the shotnoise.
    myshotnoise = ShotNoise(state=1)

    # Model the detector array.
    mydetectorarray = DetectorArray(
        name="testdetector",
        detector_array_list=mydetectorlist,
        dark_current=mydarkcurrent,
        shot_noise=myshotnoise,
        exposure_action=SummedExposure(),
    )
    # -

    # ### Simulate Several Raw Darks

    # Create a template to simulate.
    dark_template = MicadoImgCalDark(det__dit=1, det__ndit=1)

    # Create an observingblock using this template.
    dark_ob = ObservingBlock(
        name="MyDarkTest",
        instrument="MICADO",
        target="empty_sky",
        templates=[dark_template],
    )

    # Create Darks and DepersistedDarks
    mydarks = []
    mydepersisteddarks = []
    fns_dpdarks = []
    lines_bash = []
    for _ in range(5):
        # Create an empty set of pixels for each detector.
        myextensions = [
            DarkExtension(detector=det, BITPIX=-64, NAXIS=2) for det in mydetectorlist
        ]
        # Create an empty raw dark.
        mydark = Dark(
            detector=mydetectorarray,
            template=dark_template,
            observing_block=dark_ob,
            extensions=myextensions,
        )
        # Simulate the raw dark.
        mydark.make_simulation()
        mydark.save_fits()

        # Depersist the dark
        mydepersisteddark = DepersistedDark(
            raw=mydark,
            recipe=McdDetDepersist(),
        )
        # mydepersisteddark.make_proto()
        # mydepersisteddark.save_fits()
        pathd = Path(mydark.filename)
        pathdd = pathd.with_stem(pathd.stem + "_dp").with_suffix(".sof")
        fns_dpdarks.append(pathdd)
        mydepersisteddark.save_sof(filename=pathdd)
        line_bash = f"esorex {McdDetDepersist.name_eso()} {pathdd}\n"
        lines_bash.append(line_bash)

        mydarks.append(mydark)
        mydepersisteddarks.append(mydepersisteddark)

    s_mock_sof = "".join(
        f"{pathdd.with_suffix('.fits')} {DepersistedDark.name_oca()}\n"
        for pathdd in fns_dpdarks
    )
    fn_mock_sof = "micado_det_dark.sof"
    with open(fn_mock_sof, "w") as f:
        f.write(s_mock_sof)

    line_bash = f"esorex {McdDetDark.name_eso()} {fn_mock_sof}\n"
    lines_bash.append(line_bash)

    fn_bash = "run_pipeline.sh"
    with open(fn_bash, "w") as f:
        f.write("".join(lines_bash))

    # Write OCA rules for this minipipeline.
    allowed = MasterDark.get_all_input_classes()
    allowed.remove(Persistence)
    delattr(DepersistedDark, "persistence")
    rules = OCARules(allowed=allowed)
    rules.add_dataitem(MasterDark)
    with open("MCDWS_minipipeline.oca", "w") as f:
        f.write(str(rules))


if __name__ == "__main__":
    simulate_minipipeline()
