# -*- coding: utf-8 -*-
"""Test ScopeSIM installation.

Based on
https://github.com/astronomyk/ScopeSim/blob/master/scopesim/tests/tests_integrations/test_simplecado.py
"""

import numpy
import scopesim

# TODO: OBS_DIT and OBS_NDIT are renamed in newer ScopeSIM versions.
OBS_DIT = "!OBS.dit"
OBS_NDIT = "!OBS.ndit"

# DETECTOR nested dictionary
# Ideally I would like to have "kwargs" : {"filename" : "MICADO_detectors.tbl"}
# so that MICADO_detectors.tbl is the one place where the detector description
# is stored. This however is more suited to storing the info in a database
DETECTOR_YAML = {
    "object": "detector",
    "alias": "DET",
    "name": "test_detector",
    "properties": {"dit": OBS_DIT, "image_plane_id": 0},
    "effects": [
        {
            # TODO: detector_array_list is renamed in IRDB.
            "name": "detector_array_list",
            "description": "SimpleCADO detector array list",
            "class": "DetectorList",
            "kwargs": {
                "array_dict": {
                    "id": [1],
                    "pixsize": [0.015],
                    "angle": [0.0],
                    "gain": [1.0],
                    "x_cen": [0],
                    "y_cen": [0],
                    "xhw": [30.72],
                    "yhw": [30.72],
                },
                # These units are the default, so they can also be removed.
                "x_cen_unit": "mm",
                "y_cen_unit": "mm",
                "xhw_unit": "mm",
                "yhw_unit": "mm",
                "pixsize_unit": "mm",
                "angle_unit": "deg",
                "gain_unit": "electron/adu",
            },
        },
        {
            "name": "dark_current",
            "description": "SimpleCADO dark current",
            "class": "DarkCurrent",
            # [e-/s] level of dark currentSimpleCADO.yaml
            "kwargs": {"value": 0.2, "dit": OBS_DIT, "ndit": OBS_NDIT},
        },
    ],
}


OBSERVATIONS_DICT = {
    # Not yet implemented
    OBS_NDIT: 1,
    OBS_DIT: 10,
    # because optical train still need these
    "!INST.pixel_scale": 0.004,
    "!INST.plate_scale": 0.2666667,
}


def test_simplecado():
    """Test SimpleCADO, just a Dark."""
    # By using empty_sky it is ensured that we only measure the instrument.
    src = scopesim.source.source_templates.empty_sky()

    cmd = scopesim.commands.UserCommands(
        yamls=[DETECTOR_YAML],
        properties=OBSERVATIONS_DICT,
    )

    opt = scopesim.OpticalTrain(cmd)
    opt.observe(src)
    hdulists = opt.readout()
    # There can be multiple hdulists in case observations are done with
    # multiple instruments at the same time.
    hdulist = hdulists[0]
    # The first HDU never contains an image, even if there is only one.
    hdu = hdulist[1]

    # Finished - now just testing the output
    # No rogue photon torpedoes
    print(opt.image_planes[0].image)
    assert numpy.all(opt.image_planes[0].image == 0)

    # dark = 0.2 ct/s, DIT = 10s, NDIT = 1
    print(hdu.data)
    assert numpy.all(hdu.data == 2.0)
