# -*- coding: utf-8 -*-
"""Generate both RST and LaTeX DataItems tables from Python.

E.g. these:
https://docs.google.com/document/d/1QqUBEb_dv5aKi8m30sc6DQQyJqM3jGRHuO_IXBDYMUc/edit#heading=h.lqb9msvz89l2
https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500
"""
from micado.dataitems.Astromfield import Astromfield
from micado.dataitems.Bkgfield import Bkgfield
from micado.dataitems.Cam import Cam
from micado.dataitems.CoroBackground import CoroBackground
from micado.dataitems.CoroOffset import CoroOffset

# from micado.dataitems.CoroPupilmask import CoroPupilmask
from micado.dataitems.DetrendPupil import DetrendPupil
from micado.dataitems.DpsicsOpt import DpsicsOpt

# from micado.dataitems.DpsicsSci import DpsicsSci
from micado.dataitems.Nonlin import Nonlin
from micado.dataitems.ScienceAi import ScienceAi
from micado.dataitems.ScienceCoroPupil import ScienceCoroPupil
from micado.dataitems.ScienceCoroWaffle import ScienceCoroWaffle
from micado.dataitems.StarcenterPupil import StarcenterPupil
from micado.dataitems.StarcenterWaffle import StarcenterWaffle
from micado.dataitems.Stdfield import Stdfield
from micado.dataitems.Wam import Wam
from micado.dataitems.WamRef import WamRef
from micado.main.DataItem import DataItem

from micado.dataitems.Dark import Dark
from micado.dataitems.Flat import Flat
from micado.dataitems.Illum import Illum
from micado.dataitems.Science import Science

from micado.dataitems.AstromRef import AstromRef
from micado.dataitems.Persistence import Persistence
from micado.dataitems.PhotomRef import PhotomRef

from micado.dataitems.Astrom import Astrom
from micado.dataitems.Background import Background
from micado.dataitems.MasterDark import MasterDark
from micado.dataitems.MasterFlat import MasterFlat
from micado.dataitems.Detrend import Detrend
from micado.dataitems.DetrendFinal import DetrendFinal
from micado.dataitems.IllumCorr import IllumCorr
from micado.dataitems.PhotomBase import PhotomBase
from micado.dataitems.Stack import Stack

# from micado.dataitems.DetrendHCI import DetrendHCI
from micado.dataitems.DetrendWaffle import DetrendWaffle

from micado.toolbox.ditaa_from_wise import PATH_DOCS_GENERATED

CLASSES_RAW = [
    # Common with Spectroscopy.
    Dark,
    Flat,
    # Standard Imaging.
    Nonlin,
    Illum,
    Stdfield,
    Bkgfield,
    Science,
    # High Contrast Imaging.
    ScienceCoroWaffle,
    CoroOffset,
    ScienceCoroPupil,
    # CoroPupilmask,
    CoroBackground,
    # Astrometric Imaging
    ScienceAi,
    Wam,
    Cam,
    Astromfield,
]
# TODO: Create table for CLASSES_ICS.
CLASSES_ICS = [
    DpsicsOpt,
    # DpsicsSci,
    ScienceCoroWaffle,
]
CLASSES_EXTERNAL = [
    AstromRef,
    PhotomRef,
    WamRef,
]
CLASSES_PRO = [
    Astrom,
    Background,
    MasterDark,
    MasterFlat,
    Detrend,
    DetrendFinal,
    IllumCorr,
    Persistence,
    PhotomBase,
    Stack,
    # DetrendHCI,
    DetrendWaffle,
    DetrendPupil,
    StarcenterWaffle,
    StarcenterPupil,
]


def create_dataitem_tables(classes, title, subhead, filenamebase, headers, funcs_cols):
    """Create raw table."""
    cols = [
        [getattr(cls, method.__name__)() for cls in classes] for method in funcs_cols
    ]
    widths = [max([len(c) for c in col] + [8]) for col in cols]

    s_table_rst = create_dataitem_table_rst(title, subhead, headers, cols, widths)
    fn_rst = PATH_DOCS_GENERATED / filenamebase
    with open(f"{fn_rst}.rst", "w") as f:
        f.write(s_table_rst)

    s_table_tex = create_dataitem_table_tex(title, filenamebase, headers, cols)
    fn_rst = PATH_DOCS_GENERATED / filenamebase
    with open(f"{fn_rst}_imaging.tex", "w") as f:
        f.write(s_table_tex)


def create_dataitem_table_rst(title, subhead, headers, cols, widths):
    """Create RST table."""
    lines_1 = "+" + ("-" * (sum(widths) + len(widths) - 1)) + "+"
    n_cols_left = 3 if "Type" in headers else 2
    lines_2 = (
        "+"
        + ("-" * (sum(widths[:n_cols_left]) + 2))
        + "+"
        + ("-" * (sum(widths[n_cols_left:]) + len(widths[3:]) - 1))
        + "+"
    )
    lines_3 = "+" + "+".join("-" * w for w in widths) + "+"
    header_1 = f"|{title:{len(lines_3) -2 }}|"
    header_2 = (
        "|"
        + " " * (sum(widths[:n_cols_left]) + 2)
        + f"|{subhead:{sum(widths[n_cols_left:]) + len(widths[3:]) - 1}}|"
    )
    header_3 = "|" + "|".join(f"{kk:{lenk}}" for lenk, kk in zip(widths, headers)) + "|"

    rows = zip(*cols)

    srows = [
        "|"
        + "|".join(f"{kk:{lenk}}" for lenk, kk in zip(widths, row))
        + "|"
        + "\n"
        + lines_3
        for row in rows
    ]

    sstable = [
        lines_1,
        header_1,
        lines_2,
        header_2,
        lines_3,
        header_3,
        lines_3,
    ] + srows

    return "\n".join(sstable)


def create_dataitem_table_tex(title, filenamebase, headers, cols):
    """Create tex table."""

    def format_header(head):
        """Format header."""
        # Originally this prefixed 'DPR.' to the header, but this makes the
        # column broader than necessary.
        h2 = rf"\texttt{{{head}}}" if head.upper() == head else head
        return rf"\textbf{{{h2}}}"

    def format_cell(cell, i=1):
        """Format normal table cell."""
        from micado.toolbox.tex_from_python import latex_dataitem

        if i == 0:
            cell2 = cell.replace("_", "\\_")
            h2 = latex_dataitem(cell2) if cell.upper() == cell else cell2
            return h2
        else:
            h2 = rf"\texttt{{{cell}}}" if cell.upper() == cell else cell
            return h2.replace("_", "\\_")

    headers_wo_docatg = [h for h in headers if not h == "DO.CATG"]
    span_docatg_column = len(headers_wo_docatg) - 1
    size_of_description = 16 - 2 * len(headers_wo_docatg)

    scoldefs = (
        "|"
        + "|".join(
            [
                f"p{{{size_of_description}cm}}" if "escription" in h else "l"
                for h in headers_wo_docatg
            ]
        )
        + "|"
    )

    lines_header = rf"""
\begin{{longtable}}{{{scoldefs}}}
        \hline
""".strip().splitlines()

    hline = rf"        \Xcline{{1-{span_docatg_column+1}}}{{2\arrayrulewidth}}"
    cline = rf"        \cline{{2-{span_docatg_column+1}}}"
    header = (
        r"        \rowcolor{GreenYellow} "
        + " & ".join(format_header(h) for h in headers_wo_docatg)
        + r"\tabularnewline"
    )
    header_procatg = r" / \texttt{PRO.CATG}" if span_docatg_column == 1 else ""
    header_docatg = (
        rf"        \rowcolor{{GreenYellow}} & \multicolumn{{{span_docatg_column}}}{{|l|}}"
        rf"{{\textbf{{\texttt{{DO.CATG}} / \texttt{{REFLEX.CATG}}{header_procatg}}}}}"
        r" \tabularnewline"
    )

    cols_wo_docatg = [col for h, col in zip(headers, cols) if h != "DO.CATG"]
    col_docatg = [col for h, col in zip(headers, cols) if h == "DO.CATG"][0]

    rows = zip(*cols_wo_docatg)
    srows1_wo_docatg = [
        "        "
        + " & ".join(format_cell(cell, i) for i, cell in enumerate(row))
        + r" \tabularnewline"
        for row in rows
    ]
    srows1_docatg = [
        rf"        & \multicolumn{{{span_docatg_column}}}{{|l|}}{{{format_cell(catg)}}} \tabularnewline"
        for catg in col_docatg
    ]

    srows1 = [
        (hline, row_h, cline, row_d)
        for row_h, row_d in zip(srows1_wo_docatg, srows1_docatg)
    ]

    srows = [line for rh in srows1 for line in rh]

    caption = (
        f"{title} "
        "\\textbf{Name} is the name used in this document to refer to the Data Item. "
        "\\textbf{Description} is a human understandable description. "
        "\\textbf{\\texttt{DO.CATG}} / \\textbf{\\texttt{REFLEX.CATG}} are the "
        "category assigned through \\ac{OCA} classification rules by the DataOrganizer or Reflex "
        r"(see Section~\ref{subsec:OCARulesetsAndReflexWorkflowsImaging}) "
        "(NB: these exist transiently in the workflow and are not stored in the FITS file header)."
    )
    if span_docatg_column == 1:
        caption += (
            " \\textbf{\\texttt{PRO.CATG}} is the header keyword giving the OCA classification "
            "and thus from which the OCA classification is copied."
        )
    else:
        caption += (
            " \\textbf{\\texttt{CATG}} / \\textbf{\\texttt{TECH}} / \\textbf{\\texttt{TYPE}} "
            r"are the \texttt{DPR} keywords from which the \ac{OCA} classification is determined "
            "(no other header keywords are used in the classification)."
        )

    lines_footer = rf"""
        \hline
    \caption{{{caption}}}\label{{tab:{filenamebase}_imaging}}\\
\end{{longtable}}
""".strip().splitlines()

    return (
        "\n".join(lines_header + [header, cline, header_docatg] + srows + lines_footer)
        + "\n"
    )


def main():
    """Create tables."""
    headers, funcs_cols = zip(
        *[
            ("Name", DataItem.name_eso),
            # This type is always RAW: ("Type", DataItem.type_eso),
            ("Description", DataItem.description_oneline),
            ("CATG", DataItem.dpr_catg_c),
            ("TECH", DataItem.dpr_tech_c),
            ("TYPE", DataItem.dpr_type_c),
            ("DO.CATG", DataItem.name_oca),
        ]
    )
    create_dataitem_tables(
        CLASSES_RAW,
        "Summary of all Raw Instrument Data observed in imaging mode.",
        "DPR keywords",
        "dataitemsraw",
        headers,
        funcs_cols,
    )

    headers, funcs_cols = zip(
        *[
            ("Name", DataItem.name_eso),
            # This type seems redundant, it is always
            # "PRO": ("Type", DataItem.type_eso),
            ("Description", DataItem.description_oneline),
            ("DO.CATG", DataItem.name_oca),
            # TODO: Decide whether to add PRO.TYPE:
            # ("TYPE", DataItem.dpr_type_c),
        ]
    )
    create_dataitem_tables(
        CLASSES_EXTERNAL,
        "Summary of all External DataItems for imaging.",
        "PRO keywords",
        "dataitemsexternal",
        headers,
        funcs_cols,
    )
    create_dataitem_tables(
        CLASSES_PRO,
        "Summary of all Intermediate and Final Data for imaging.",
        "PRO keywords",
        "dataitemspro",
        headers,
        funcs_cols,
    )


if __name__ == "__main__":
    main()
